# Natural Language Toolkit: modals

import nltk, math
modals = ['can', 'could', 'may', 'might', 'must', 'will']

def modal_counts(tokens):
    return nltk.FreqDist(word for word in tokens if word in modals)

# just the most frequent modal verb
def modal_features1(tokens):
    return dict(most_frequent_modal = model_counts(tokens).max())

# one feature per verb, set to True if the verb occurs more than once
def modal_features2(tokens):
    fd = modal_counts(tokens)
    return dict( (word,(fd[word]>1)) for word in modals)

# one feature per verb, with a small number of scalar values
def modal_features3(tokens):
    fd = modal_counts(tokens)
    features = {}
    for word in modals:
        try:
            features[word] = int(-math.log10(float(fd[word])/len(tokens)))
        except OverflowError:
            features[word] = 1000
    return features

# 4 bins per verb based on frequency
def modal_features4(tokens):
    fd = modal_counts(tokens)
    features = {}
    for word in modals:
        freq = float(fd[word])/len(tokens)
        for logfreq in range(3,7):
            features["%s(%d)" % (word, logfreq)] = (freq < 10**(-logfreq))
    return features

